# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Tests for the base directory implementation."""

from __future__ import unicode_literals, print_function

import os

from dirspec import basedir
from dirspec.tests import BaseTestCase


class BasedirTestCase(BaseTestCase):
    """Tests for XDG Base Directory paths implementation."""

    def test_cache_home(self):
        """Test that XDG_CACHE_HOME is handled correctly."""
        self.tweak_env('XDG_CACHE_HOME',
                       os.path.abspath(os.path.join(os.getcwd(),
                                                    '_trial_temp',
                                                    'cache')))
        self.assertEqual(os.environ['XDG_CACHE_HOME'].encode('utf-8'),
                         basedir.get_xdg_cache_home())

    def test_config_dirs(self):
        """Test that XDG_CONFIG_HOME is handled correctly."""
        self.tweak_env('XDG_CONFIG_HOME',
                       os.path.abspath(os.path.join(os.getcwd(),
                                                    '_trial_temp',
                                                    'config')))
        self.tweak_env('XDG_CONFIG_DIRS', os.pathsep.join(['etc']))
        self.assertEqual([os.environ['XDG_CONFIG_HOME'].encode('utf-8'),
                          b'etc'],
                         basedir.get_xdg_config_dirs())

    def test_config_home(self):
        """Test that XDG_CONFIG_DIRS is handled correctly."""
        self.tweak_env('XDG_CONFIG_HOME',
                       os.path.abspath(os.path.join(os.getcwd(),
                                                    '_trial_temp',
                                                    'config')))
        self.assertEqual(os.environ['XDG_CONFIG_HOME'].encode('utf-8'),
                         basedir.get_xdg_config_home())

    def test_data_dirs(self):
        """Test that XDG_DATA_HOME is handled correctly."""
        self.tweak_env('XDG_DATA_HOME',
                       os.path.abspath(os.path.join(os.getcwd(),
                                                    '_trial_temp',
                                                    'xdg_data')))
        self.tweak_env('XDG_DATA_DIRS', os.pathsep.join(['foo', 'bar']))
        self.assertEqual([os.environ['XDG_DATA_HOME'].encode('utf-8'),
                          b'foo', b'bar'],
                         basedir.get_xdg_data_dirs())

    def test_data_home(self):
        """Test that XDG_DATA_HOME is handled correctly."""
        self.tweak_env('XDG_DATA_HOME',
                       os.path.abspath(os.path.join(os.getcwd(),
                                                    '_trial_temp',
                                                    'xdg_data')))
        self.assertEqual(os.environ['XDG_DATA_HOME'].encode('utf-8'),
                         basedir.get_xdg_data_home())

    def test_default_cache_home(self):
        """Ensure default values work correctly."""
        self.tweak_env('XDG_CACHE_HOME', None)
        expected = b'/blah'
        self.patch(basedir, 'default_cache_home', expected)
        self.assertFalse(os.environ.get('XDG_CACHE_HOME', False))
        self.assertEqual(basedir.get_xdg_cache_home(), expected)

    def test_default_config_dirs(self):
        """Ensure default values work correctly."""
        self.tweak_env('XDG_CONFIG_DIRS', None)
        self.tweak_env('XDG_CONFIG_HOME', None)
        expected = b'/blah'
        self.patch(basedir, 'default_config_home', expected)
        self.patch(basedir, 'default_config_path', '')
        self.assertFalse(os.environ.get('XDG_CONFIG_DIRS', False))
        self.assertFalse(os.environ.get('XDG_CONFIG_HOME', False))
        self.assertEqual(basedir.get_xdg_config_dirs(), [expected, b''])

    def test_default_config_home(self):
        """Ensure default values work correctly."""
        self.tweak_env('XDG_CONFIG_HOME', None)
        expected = b'/blah'
        self.patch(basedir, 'default_config_home', expected)
        self.assertFalse(os.environ.get('XDG_CONFIG_HOME', False))
        self.assertEqual(basedir.get_xdg_config_home(), expected)

    def test_default_data_dirs(self):
        """Ensure default values work correctly."""
        self.tweak_env('XDG_DATA_DIRS', None)
        self.tweak_env('XDG_DATA_HOME', None)
        expected = b'/blah'
        self.patch(basedir, 'default_data_home', expected)
        self.patch(basedir, 'default_data_path', '')
        self.assertFalse(os.environ.get('XDG_DATA_DIRS', False))
        self.assertFalse(os.environ.get('XDG_DATA_HOME', False))
        self.assertEqual(basedir.get_xdg_data_dirs(), [expected, b''])

    def test_default_data_home(self):
        """Ensure default values work correctly."""
        self.tweak_env('XDG_DATA_HOME', None)
        expected = b'/blah'
        self.patch(basedir, 'default_data_home', expected)
        self.assertFalse(os.environ.get('XDG_DATA_HOME', False))
        self.assertEqual(basedir.get_xdg_data_home(), expected)

    def test_xdg_cache_home_is_utf8_bytes(self):
        """The returned path is bytes."""
        actual = basedir.xdg_cache_home
        self.assert_utf8_bytes(actual)

    def test_xdg_config_home_is_utf8_bytes(self):
        """The returned path is bytes."""
        actual = basedir.xdg_config_home
        self.assert_utf8_bytes(actual)

    def test_xdg_config_dirs_are_bytes(self):
        """The returned path is bytes."""
        result = basedir.xdg_config_dirs
        for actual in result:
            self.assert_utf8_bytes(actual)

    def test_xdg_data_home_is_utf8_bytes(self):
        """The returned path is bytes."""
        actual = basedir.xdg_data_home
        self.assert_utf8_bytes(actual)

    def test_xdg_data_dirs_are_bytes(self):
        """The returned path is bytes."""
        result = basedir.xdg_data_dirs
        for actual in result:
            self.assert_utf8_bytes(actual)

    def test_load_config_paths_filter(self):
        """Since those folders don't exist, this should be empty."""
        self.assertEqual(list(basedir.load_config_paths("x")), [])

    def test_save_config_path(self):
        """The path should end with xdg_config/x (respecting the separator)."""
        self.tweak_env('XDG_CONFIG_HOME', 'config_home')
        self.patch(os, "makedirs", lambda *args: None)
        result = basedir.save_config_path("x")
        self.assertEqual(result.decode('utf-8').split(os.sep)[-2:],
                         ['config_home', 'x'])



# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Utilities for multiplatform support of XDG directory handling."""

from __future__ import unicode_literals, print_function

import errno
import os
import sys

__all__ = ['user_home',
           'default_cache_home',
           'default_config_home',
           'default_config_path',
           'default_data_home',
           'default_data_path',
           'get_env_path',
           'get_program_path',
           'unicode_path',
           ]


def _get_exe_path_frozen_win32(exe_name):
    """Get path to the helper .exe on packaged windows."""
    # all the .exes are in the same place on windows:
    cur_exec_path = os.path.abspath(sys.executable)
    exe_dir = os.path.dirname(cur_exec_path)
    return os.path.join(exe_dir, exe_name + ".exe")


def _get_exe_path_frozen_darwin(exe_name, app_names):
    """Get path to the sub-app executable on packaged darwin."""

    sub_app_name = app_names[exe_name]
    main_app_dir = "".join(__file__.partition(".app")[:-1])
    main_app_resources_dir = os.path.join(main_app_dir,
                                          "Contents",
                                          "Resources")
    exe_bin = os.path.join(main_app_resources_dir,
                           sub_app_name,
                           "Contents", "MacOS",
                           exe_name)
    return exe_bin


def get_program_path(program_name, *args, **kwargs):
    """Given a program name, returns the path to run that program.

    Raises OSError if the program is not found.

    :param program_name: The name of the program to find. For darwin and win32
        platforms, the behavior is changed slightly, when sys.frozen is set,
        to look in the packaged program locations for the program.
    :param search_dirs: A list of directories to look for the program in. This
        is only available as a keyword argument.
    :param app_names: A dict of program names mapped to sub-app names. Used
        for discovering paths in embedded .app bundles on the darwin platform.
        This is only available as a keyword argument.
    :return: The path to the discovered program.
    """
    search_dirs = kwargs.get('fallback_dirs', None)
    app_names = kwargs.get('app_names', None)

    if getattr(sys, "frozen", None) is not None:
        if sys.platform == 'win32':
            program_path = _get_exe_path_frozen_win32(program_name)
        elif sys.platform == 'darwin':
            program_path = _get_exe_path_frozen_darwin(program_name,
                                                       app_names)
        else:
            raise Exception("Unsupported platform for frozen execution: %r" %
                            sys.platform)
    else:
        if search_dirs is not None:
            for dirname in search_dirs:
                program_path = os.path.join(dirname, program_name)
                if os.path.exists(program_path):
                    return program_path
        else:
            # Check in normal system $PATH, if no fallback dirs specified
            from distutils.spawn import find_executable
            program_path = find_executable(program_name)

    if program_path is None or not os.path.exists(program_path):
        raise OSError(errno.ENOENT,
                      "Could not find executable %r" % program_name)

    return program_path


def get_env_path(key, default):
    """Get a UTF-8 encoded path from an environment variable."""
    if key in os.environ:
        # on windows, environment variables are mbcs bytes
        # so we must turn them into utf-8 Syncdaemon paths
        try:
            path = os.environb.get(key.encode('utf-8'))
        except AttributeError:
            path = os.environ[key]
        return path.decode(sys.getfilesystemencoding()).encode('utf-8')
    else:
        if not isinstance(default, bytes):
            return default.encode('utf-8')
        return default


def unicode_path(utf8path):
    """Turn an utf8 path into a unicode path."""
    if isinstance(utf8path, bytes):
        return utf8path.decode("utf-8")
    return utf8path


def get_special_folders():
    """ Routine to grab all the Windows Special Folders locations.

    If successful, returns dictionary
    of shell folder locations indexed on Windows keyword for each;
    otherwise, returns an empty dictionary.
    """
    # pylint: disable=W0621, F0401, E0611
    special_folders = {}

    if sys.platform == 'win32':
        from win32com.shell import shell, shellcon
        # CSIDL_LOCAL_APPDATA = C:\Users\<username>\AppData\Local
        # CSIDL_PROFILE = C:\Users\<username>
        # CSIDL_COMMON_APPDATA = C:\ProgramData
        # More information on these constants at
        # http://msdn.microsoft.com/en-us/library/bb762494

        # per http://msdn.microsoft.com/en-us/library/windows/desktop/bb762181,
        # SHGetFolderPath is deprecated, replaced by SHGetKnownFolderPath
        # (http://msdn.microsoft.com/en-us/library/windows/desktop/bb762188)
        get_path = lambda name: shell.SHGetFolderPath(
            0, getattr(shellcon, name), None, 0).encode('utf8')
        special_folders['Personal'] = get_path("CSIDL_PROFILE")
        special_folders['Local AppData'] = get_path("CSIDL_LOCAL_APPDATA")
        special_folders['AppData'] = os.path.dirname(
            special_folders['Local AppData'])
        special_folders['Common AppData'] = get_path("CSIDL_COMMON_APPDATA")

    return special_folders


# pylint: disable=C0103
if sys.platform == 'win32':
    special_folders = get_special_folders()
    user_home = special_folders['Personal']
    default_config_path = special_folders['Common AppData']
    default_config_home = special_folders['Local AppData']
    default_data_path = os.path.join(default_config_path, b'xdg')
    default_data_home = os.path.join(default_config_home, b'xdg')
    default_cache_home = os.path.join(default_data_home, b'cache')
elif sys.platform == 'darwin':
    user_home = os.path.expanduser(b'~')
    default_cache_home = os.path.join(user_home, b'Library', b'Caches')
    default_config_path = b'/Library/Preferences:/etc/xdg'
    default_config_home = os.path.join(user_home, b'Library', b'Preferences')
    default_data_path = b':'.join([b'/Library/Application Support',
                                  b'/usr/local/share',
                                  b'/usr/share'])
    default_data_home = os.path.join(user_home, b'Library',
                                     b'Application Support')
else:
    user_home = os.path.expanduser(b'~')
    default_cache_home = os.path.join(user_home,
                                      b'.cache')
    default_config_path = b'/etc/xdg'
    default_config_home = os.path.join(user_home,
                                       b'.config')
    default_data_path = b'/usr/local/share:/usr/share'
    default_data_home = os.path.join(user_home,
                                     b'.local', b'share')



# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""XDG Base Directory paths."""

from __future__ import unicode_literals, print_function

import os

from dirspec.utils import (default_cache_home,
                           default_config_path, default_config_home,
                           default_data_path, default_data_home,
                           get_env_path, unicode_path)

__all__ = ['xdg_cache_home',
           'xdg_config_home',
           'xdg_data_home',
           'xdg_config_dirs',
           'xdg_data_dirs',
           'load_config_paths',
           'load_data_paths',
           'load_first_config',
           'save_config_path',
           'save_data_path',
           ]


def get_xdg_cache_home():
    """Get the path for XDG cache directory in user's HOME."""
    return get_env_path('XDG_CACHE_HOME', default_cache_home)


def get_xdg_config_home():
    """Get the path for XDG config directory in user's HOME."""
    return get_env_path('XDG_CONFIG_HOME', default_config_home)


def get_xdg_data_home():
    """Get the path for XDG data directory in user's HOME."""
    return get_env_path('XDG_DATA_HOME', default_data_home)


def get_xdg_config_dirs():
    """Get the paths for the XDG config directories."""
    result = [get_xdg_config_home()]
    result.extend([x.encode('utf-8') for x in get_env_path(
        'XDG_CONFIG_DIRS',
        default_config_path).decode('utf-8').split(os.pathsep)])
    return result


def get_xdg_data_dirs():
    """Get the paths for the XDG data directories."""
    result = [get_xdg_data_home()]
    result.extend([x.encode('utf-8') for x in get_env_path(
        'XDG_DATA_DIRS',
        default_data_path).decode('utf-8').split(os.pathsep)])
    return result


def load_config_paths(*resource):
    """Iterator of configuration paths.

    Return an iterator which gives each directory named 'resource' in
    the configuration search path. Information provided by earlier
    directories should take precedence over later ones (ie, the user's
    config dir comes first).
    """
    resource = os.path.join(*resource)
    assert not resource.startswith('/')
    for config_dir in get_xdg_config_dirs():
        path = os.path.join(config_dir, resource.encode('utf-8'))
        # access the file system always with unicode
        # to properly behave in some operating systems
        if os.path.exists(unicode_path(path)):
            yield path


def load_data_paths(*resource):
    """Iterator of data paths.

    Return an iterator which gives each directory named 'resource' in
    the stored data search path. Information provided by earlier
    directories should take precedence over later ones.
    """
    resource = os.path.join(*resource)
    assert not resource.startswith('/')
    for data_dir in get_xdg_data_dirs():
        path = os.path.join(data_dir, resource.encode('utf-8'))
        # access the file system always with unicode
        # to properly behave in some operating systems
        if os.path.exists(unicode_path(path)):
            yield path


def load_first_config(*resource):
    """Returns the first result from load_config_paths, or None if nothing
    is found to load.
    """
    for path in load_config_paths(*resource):
        return path
    return None


def save_config_path(*resource):
    """Path to save configuration.

    Ensure $XDG_CONFIG_HOME/<resource>/ exists, and return its path.
    'resource' should normally be the name of your application. Use this
    when SAVING configuration settings. Use the xdg_config_dirs variable
    for loading.
    """
    resource = os.path.join(*resource)
    assert not resource.startswith('/')
    path = os.path.join(get_xdg_config_home(), resource.encode('utf-8'))
    # access the file system always with unicode
    # to properly behave in some operating systems
    if not os.path.isdir(unicode_path(path)):
        os.makedirs(unicode_path(path), 0o700)
    return path


def save_data_path(*resource):
    """Path to save data.

    Ensure $XDG_DATA_HOME/<resource>/ exists, and return its path.
    'resource' should normally be the name of your application. Use this
    when STORING a resource. Use the xdg_data_dirs variable for loading.
    """
    resource = os.path.join(*resource)
    assert not resource.startswith('/')
    path = os.path.join(get_xdg_data_home(), resource.encode('utf-8'))
    # access the file system always with unicode
    # to properly behave in some operating systems
    if not os.path.isdir(unicode_path(path)):
        os.makedirs(unicode_path(path), 0o700)
    return path


# pylint: disable=C0103
xdg_cache_home = get_xdg_cache_home()
xdg_config_home = get_xdg_config_home()
xdg_data_home = get_xdg_data_home()

xdg_config_dirs = [x for x in get_xdg_config_dirs() if x]
xdg_data_dirs = [x for x in get_xdg_data_dirs() if x]
# pylint: disable=C0103



# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
""""dirspec tests."""

from __future__ import unicode_literals, print_function

import os

from operator import setitem
from testtools.testcase import TestCase


class BaseTestCase(TestCase):
    """Base test case for dirspect tests."""

    def assert_utf8_bytes(self, value):
        """Check that 'value' is a bytes sequence encoded with utf-8."""
        self.assertIsInstance(value, bytes)
        try:
            value.decode('utf-8')
        except UnicodeError:
            self.fail('%r should be a utf8 encoded string.' % value)

    def tweak_env(self, envvar, value):
        """Tweak the environment variable %var to %value.

        Restore the old value when finished.
        """
        old_val = os.environ.get(envvar, None)

        if old_val is None:
            self.addCleanup(os.environ.pop, envvar, None)
        else:
            self.addCleanup(setitem, os.environ, envvar, old_val)
        if value is None:
            os.environ.pop(envvar, None)
        else:
            os.environ[envvar] = value



# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Tests for utilities for the base directory implementation."""

from __future__ import unicode_literals, print_function

import os
import sys

from dirspec import basedir, utils as dirutils
from dirspec.utils import (get_env_path, get_special_folders,
                           user_home, get_program_path)
from dirspec.tests import BaseTestCase
from testtools.testcase import skip


class UtilsTestCase(BaseTestCase):
    """Test for the multiplatform directory utilities."""

    def test_user_home_is_utf8_bytes(self):
        """The returned path is bytes."""
        actual = user_home
        self.assert_utf8_bytes(actual)


class FakeShellConModule(object):
    """Override CSIDL_ constants."""

    CSIDL_PROFILE = 0
    CSIDL_LOCAL_APPDATA = 1
    CSIDL_COMMON_APPDATA = 2


class FakeShellModule(object):

    """Fake Shell Module."""

    def __init__(self):
        """Set the proper mapping between CSIDL_ consts."""
        self.values = {
            0: 'c:\\path\\to\\users\\home',
            1: 'c:\\path\\to\\users\\home\\appData\\local',
            2: 'c:\\programData',
        }

    # pylint: disable=C0103
    def SHGetFolderPath(self, dummy0, shellconValue, dummy2, dummy3):
        """Override SHGetFolderPath functionality."""
        return self.values[shellconValue]
    # pylint: enable=C0103


class TestBaseDirectoryWindows(BaseTestCase):
    """Tests for the BaseDirectory module."""

    def test_get_special_folders(self):
        """Make sure we can import the platform module."""
        if sys.platform != 'win32':
            self.skipTest('Win32 is required for this test.')

        import win32com.shell
        shell_module = FakeShellModule()
        self.patch(win32com.shell, "shell", shell_module)
        self.patch(win32com.shell, "shellcon", FakeShellConModule())
        special_folders = get_special_folders()
        self.assertTrue('Personal' in special_folders)
        self.assertTrue('Local AppData' in special_folders)
        self.assertTrue('AppData' in special_folders)
        self.assertTrue('Common AppData' in special_folders)

        self.assertEqual(
            special_folders['Personal'],
            shell_module.values[FakeShellConModule.CSIDL_PROFILE])
        self.assertEqual(
            special_folders['Local AppData'],
            shell_module.values[FakeShellConModule.CSIDL_LOCAL_APPDATA])
        self.assertTrue(
            special_folders['Local AppData'].startswith(
                special_folders['AppData']))
        self.assertEqual(
            special_folders['Common AppData'],
            shell_module.values[FakeShellConModule.CSIDL_COMMON_APPDATA])

        for val in special_folders.itervalues():
            self.assertIsInstance(val, str)
            val.encode('utf-8')

    def test_get_data_dirs(self):
        """Check thet get_data_dirs uses pathsep correctly."""
        bad_sep = str(filter(lambda x: x not in os.pathsep, ":;"))
        dir_list = ["A", "B", bad_sep, "C"]
        self.tweak_env('XDG_DATA_DIRS', os.pathsep.join(dir_list))
        dirs = basedir.get_xdg_data_dirs()[1:]
        self.assertEqual(dirs, [x.encode('utf-8') for x in dir_list])

    def test_get_config_dirs(self):
        """Check thet get_data_dirs uses pathsep correctly."""
        bad_sep = str(filter(lambda x: x not in os.pathsep, ":;"))
        dir_list = ["A", "B", bad_sep, "C"]
        self.tweak_env('XDG_CONFIG_DIRS', os.pathsep.join(dir_list))
        dirs = basedir.get_xdg_config_dirs()[1:]
        self.assertEqual(dirs, [x.encode('utf-8') for x in dir_list])

    def unset_fake_environ(self, key):
        """Unset (and restore) a fake environ variable."""
        if key in os.environ:
            current_value = os.environ[key]
            self.addCleanup(os.environ.__setitem__, key, current_value)
            del(os.environ[key])

    @skip('UnicodeEncodeError: bug #907053')
    def test_get_env_path_var(self):
        """Test that get_env_path transforms an env var."""
        fake_path = 'C:\\Users\\Ñandú'
        fake_env_var = 'FAKE_ENV_VAR'

        mbcs_path = fake_path.encode(sys.getfilesystemencoding())

        self.tweak_env(fake_env_var, str(mbcs_path))
        self.assertEqual(get_env_path(fake_env_var, "unexpected"), fake_path)

    @skip('UnicodeEncodeError: bug #907053')
    def test_get_env_path_no_var(self):
        """Test that get_env_path returns the default when env var not set."""
        fake_path = "C:\\Users\\Ñandú"
        fake_env_var = "fake_env_var"
        default = fake_path.encode(sys.getfilesystemencoding())

        self.unset_fake_environ(fake_env_var)
        self.assertEqual(get_env_path(fake_env_var, default), default)


class ProgramPathBaseTestCase(BaseTestCase):
    """Base class for testing the executable finder."""

    def setUp(self):
        """Set up fake modules."""
        super(ProgramPathBaseTestCase, self).setUp()
        self.patch(os.path, "exists", lambda x: True)


class UnfrozenSrcTestCase(ProgramPathBaseTestCase):
    """Test non-linux path discovery."""

    def setUp(self):
        super(UnfrozenSrcTestCase, self).setUp()
        self.patch(sys, "platform", "darwin")

    def test_unfrozen_dev_toplevel(self):
        """Not frozen, return path to bin dir."""
        path = get_program_path("foo", fallback_dirs=['/path/to/bin'])
        self.assertEquals(path, os.path.join("/path/to/bin", "foo"))

    def test_unfrozen_dev_toplevel_raises_nopath(self):
        """Not frozen, raise OSError when the path doesn't exist."""
        self.patch(os.path, "exists", lambda x: False)
        self.assertRaises(OSError, get_program_path, "foo")


class DarwinPkgdTestCase(ProgramPathBaseTestCase):
    """Test cmdline for running packaged on darwin."""

    def setUp(self):
        """SetUp to mimic frozen darwin."""
        super(DarwinPkgdTestCase, self).setUp()
        self.patch(sys, "platform", "darwin")
        sys.frozen = True

        self.darwin_app_names = {"foo": "Foo.app"}

    def tearDown(self):
        """tearDown, Remove frozen attr"""
        del sys.frozen
        super(DarwinPkgdTestCase, self).tearDown()

    def test_darwin_pkgd(self):
        """Return sub-app path on darwin when frozen."""
        path = get_program_path("foo", app_names=self.darwin_app_names)
        expectedpath = "%s%s" % (
            dirutils.__file__,
            os.path.sep + os.path.join('Contents', 'Resources', 'Foo.app',
                                       'Contents', 'MacOS', 'foo'))
        self.assertEquals(path, expectedpath)

    def test_darwin_pkgd_raises_on_no_appnames(self):
        """Raises TypeError when no app_names dict is in the kwargs."""
        self.assertRaises(TypeError, get_program_path, "foo")

    def test_darwin_pkgd_raises_nopath(self):
        """Frozen, raise OSError when the path doesn't exist."""
        self.patch(os.path, "exists", lambda x: False)
        self.assertRaises(OSError, get_program_path, "foo",
                          app_names=self.darwin_app_names)


class Win32PkgdTestCase(ProgramPathBaseTestCase):
    """Test cmdline for running packaged on windows."""

    def setUp(self):
        """SetUp to mimic frozen windows."""
        super(Win32PkgdTestCase, self).setUp()
        self.patch(sys, "platform", "win32")
        sys.frozen = True

    def tearDown(self):
        """tearDown, Remove frozen attr"""
        del sys.frozen
        super(Win32PkgdTestCase, self).tearDown()

    def test_windows_pkgd(self):
        """Return sub-app path on windows when frozen."""

        self.patch(sys, "executable", os.path.join("C:\\path", "to",
                                                   "current.exe"))
        # patch abspath to let us run this tests on non-windows:
        self.patch(os.path, "abspath", lambda x: x)
        path = get_program_path("foo", None)
        expectedpath = os.path.join("C:\\path", "to", "foo.exe")
        self.assertEquals(path, expectedpath)

    def test_windows_pkgd_raises_nopath(self):
        """Frozen, raise OSError when the path doesn't exist."""
        self.patch(os.path, "exists", lambda x: False)
        self.assertRaises(OSError, get_program_path, "foo")


class PosixTestCase(ProgramPathBaseTestCase):
    """Test cmdline for running on linux."""

    def setUp(self):
        """SetUp to mimic linux2."""
        super(PosixTestCase, self).setUp()
        self.patch(sys, "platform", "linux2")

    def test_linux_src_relative_path_exists(self):
        """linux, return source relative path if it exists."""
        path = get_program_path("foo", fallback_dirs=['/path/to/bin'])
        expectedpath = os.path.join("/path/to/bin", "foo")
        self.assertEquals(path, expectedpath)

    def test_linux_no_src_relative_path(self):
        """raise if no src rel path."""
        self.patch(os.path, "exists", lambda x: False)
        self.assertRaises(OSError, get_program_path, "foo")



# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""dirspec package."""
